#######################################################################
##### Part of the VortexGE Configuration Script                   #####
##### Copyright (C) 2004-2006 Aloysius Indrayanto                 #####
##### GNU Lesser General Public License                           #####
##### ----------------------------------------------------------- #####
##### EDITING THIS FILE ERRONEOUSLY MAY CAUSE THIS SCRIPT TO FAIL #####
##### ----------------------------------------------------------- #####
##### This program is distributed WITHOUT ANY WARRANTY.           #####
##### See the GNU Lesser General Public License for more details. #####
#######################################################################

#######################################################################
##### Functions for Detecting CPU Models using /proc/cpuinfo
#######################################################################
DetectCPUModel_Intel()
{
  local __cpu_vend__=$1
  local __cpu_type__=$2
  local __cpu_flag__=$3

  ##### Detect for Core Series CPUs
  if [[ `echo $__cpu_type__ | grep -e '[TLU]1[0123456789][05]0.*@.*G'` != '' ]]; then
    Processor=$CPU_IntelCoreSolo
  elif [[ `echo $__cpu_type__ | grep -e '[TLU]2[0123456789][05]0.*@.*G'` != '' ]]; then
    Processor=$CPU_IntelCoreDuo
  elif [[ `echo $__cpu_type__ | grep -e '[XETLU][567][0123456789]00.*@.*G'` != '' ]]; then
    Processor=$CPU_IntelCore2Duo # Quad core CPUs 'QX6.00' are still based on the Core2Duo technology
  ##### Detect for Celeron CPUs
  elif [[ `echo $__cpu_type__ | grep 'Celeron'` != '' ]]; then
    if [[ `echo $__cpu_type__ | grep ' D '`  != '' ]] ||
       [[ `echo $__cpu_flag__ | grep 'pni'`  != '' ]] ||
       [[ `echo $__cpu_flag__ | grep 'sse3'` != '' ]]; then
      Processor=$CPU_IntelCeleronD
    elif [[ `echo $__cpu_flag__ | grep 'sse2'` != '' ]]; then
      if [[ `echo $__cpu_type__ | grep ' M '` != '' ]]; then
        Processor=$CPU_IntelCeleronM
      else
        Processor=$CPU_IntelCeleron3
      fi
    elif [[ `echo $__cpu_flag__ | grep 'sse'` != '' ]]; then
      Processor=$CPU_IntelCeleron2
    else
      Processor=$CPU_IntelCeleron
    fi
  ##### Detect for Pentium Xeon CPUs
  elif [[ `echo $__cpu_type__ | grep 'Xeon'` != '' ]] ||
       [[ `echo $__cpu_type__ | grep 'XEON'` != '' ]]; then
    if [[ `echo $__cpu_flag__ | grep 'sse2'` != '' ]]; then
      Processor=$CPU_IntelP4Xeon
    elif [[ `echo $__cpu_flag__ | grep 'sse'` != '' ]]; then
      Processor=$CPU_IntelP3Xeon
    else
      Processor=$CPU_IntelP2Xeon
    fi
  ##### Detect for Pentium CPUs
  elif [[ `echo $__cpu_type__ | grep 'Pentium'` != '' ]]; then
    if [[ `echo $__cpu_type__ | grep ' D '` != '' ]]; then
      Processor=$CPU_IntelPentiumD
    elif [[ `echo $__cpu_type__ | grep ' 4 '` != '' ]]; then
      if [[ `echo $__cpu_flag__ | grep 'pni'`  != '' ]] ||
         [[ `echo $__cpu_flag__ | grep 'sse3'` != '' ]]; then
        if [[ `uname -m` == x86_64 ]]; then
          Processor=$CPU_IntelNocona
        else
          Processor=$CPU_IntelPrescott
        fi
      else
        Processor=$CPU_IntelPentium4
      fi
    elif [[ `echo $__cpu_type__ | grep ' M '` != '' ]]; then
      Processor=$CPU_IntelPentiumM
    elif [[ `echo $__cpu_type__ | grep 'III'` != '' ]]; then
      Processor=$CPU_IntelPentiumIII
    elif [[ `echo $__cpu_type__ | grep 'II'` != '' ]]; then
      Processor=$CPU_IntelPentiumII
    elif [[ `echo $__cpu_type__ | grep 'Pro'` != '' ]]; then
      Processor=$CPU_IntelPentiumPro
    elif [[ `echo $__cpu_type__ | grep 'MMX'` != '' ]]; then
      Processor=$CPU_IntelPentiumMMX
    else
      if [[ `echo $__cpu_flag__ | grep 'pni'`  != '' ]] ||
         [[ `echo $__cpu_flag__ | grep 'sse3'` != '' ]]; then
        Processor=$CPU_IntelPrescott
      elif [[ `echo $__cpu_flag__ | grep 'sse2'` != '' ]]; then
        Processor=$CPU_IntelPentium4
      elif [[ `echo $__cpu_flag__ | grep 'sse'` != '' ]]; then
        Processor=$CPU_IntelPentiumIII
      elif [[ `echo $__cpu_flag__ | grep 'mmx'` != '' ]]; then
        Processor=$CPU_IntelPentiumMMX
      else
        Processor=$CPU_IntelPentium
      fi
    fi
  elif [[ `echo $__cpu_type__ | grep 'Centrino'` != '' ]]; then
    Processor=$CPU_IntelPentiumM
  elif [[ `echo $__cpu_type__ | grep 'Prescott'` != '' ]]; then
    Processor=$CPU_IntelPrescott
  elif [[ `echo $__cpu_type__ | grep 'Nocona'` != '' ]]; then
    Processor=$CPU_IntelNocona
  fi
}

DetectCPUModel_AMD()
{
  local __cpu_vend__=$1
  local __cpu_type__=$2
  local __cpu_flag__=$3

  ##### Detect for K6 CPUs
  if [[ `echo $__cpu_type__ | grep 'K6'` != '' ]]; then
    if [[ `echo $__cpu_type__ | grep 'III'` != '' ]] ||
       [[ `echo $__cpu_type__ | grep '3D+'` != '' ]]; then
      Processor=$CPU_AMDK6III
    elif [[ `echo $__cpu_type__ | grep 'II'` != '' ]] ||
         [[ `echo $__cpu_type__ | grep '3D'` != '' ]]; then
      Processor=$CPU_AMDK6II
    else
      Processor=$CPU_AMDK6
    fi
  ##### Detect for Athlon CPUs
  elif [[ `echo $__cpu_type__ | grep 'K7'` != '' ]]; then
    Processor=$CPU_AMDAthlon
  elif [[ `echo $__cpu_type__ | grep 'K8'` != '' ]]; then
    Processor=$CPU_AMDK8
  elif [[ `echo $__cpu_type__ | grep 'Athlon'` != '' ]]; then
    if [[ `echo $__cpu_type__ | grep ' X2 '` != '' ]]; then
      Processor=$CPU_AMDAthlonX2
    elif [[ `echo $__cpu_type__ | grep 'FX'` != '' ]]; then
      Processor=$CPU_AMDAthlonFX
    elif [[ `echo $__cpu_type__ | grep ' 64 '` != '' ]]; then
      Processor=$CPU_AMDAthlon64
    elif [[ `echo $__cpu_type__ | grep 'MP'` != '' ]]; then
      Processor=$CPU_AMDAthlonMP
    elif [[ `echo $__cpu_type__ | grep 'XP'` != '' ]]; then
      Processor=$CPU_AMDAthlonXP
    elif [[ `echo $__cpu_type__ | grep ' 4 '` != '' ]]; then
      Processor=$CPU_AMDAthlon4
    elif [[ `echo $__cpu_type__ | grep 'Thunderbird'` != '' ]] ||
         [[ `echo $__cpu_type__ | grep 'TBird'`       != '' ]] ||
         [[ `echo $__cpu_type__ | grep 'Tbird'`       != '' ]] ||
         [[ `echo $__cpu_type__ | grep 'tbird'`       != '' ]]; then
      Processor=$CPU_AMDAthlonTBird
    else
      Processor=$CPU_AMDAthlon
    fi
  ##### Detect for other CPUs
  elif [[ `echo $__cpu_type__ | grep 'Opteron'` != '' ]]; then
    Processor=$CPU_AMDOpteron
  elif [[ `echo $__cpu_type__ | grep 'Turion'` != '' ]]; then
    Processor=$CPU_AMDTurion
  elif [[ `echo $__cpu_type__ | grep 'Sempron'` != '' ]]; then
    Processor=$CPU_AMDSempron
  elif [[ `echo $__cpu_type__ | grep 'Duron'` != '' ]]; then
    Processor=$CPU_AMDDuron
  fi
}

DetectCPUModel_Others()
{
  local __cpu_vend__=$1
  local __cpu_type__=$2
  local __cpu_flag__=$3

  ##### Detect for IDT's and VIA's CPUs
  if [[ `echo $__cpu_vend__ | grep 'IDT'`          != '' ]] ||
     [[ `echo $__cpu_vend__ | grep 'VIA'`          != '' ]] ||
     [[ `echo $__cpu_vend__ | grep 'Via'`          != '' ]] ||
     [[ `echo $__cpu_vend__ | grep 'CentaurHauls'` != '' ]]; then
    if [[ `echo $__cpu_type__ | grep 'C6-2'` != '' ]]; then
      Processor=$CPU_IDTWinChip2
    elif [[ `echo $__cpu_type__ | grep 'C6'` != '' ]]; then
      if [[ `echo $__cpu_type__ | grep '+'` != '' ]]; then
        Processor=$CPU_IDTWinChip2
      else
        Processor=$CPU_IDTWinChipC6
      fi
    elif [[ `echo $__cpu_type__ | grep 'WinChip'` != '' ]]; then
      if [[ `echo $__cpu_type__ | grep ' 2 '` != '' ]]; then
        Processor=$CPU_IDTWinChip2
      fi
    elif [[ `echo $__cpu_type__ | grep 'C3-2'`     != '' ]]
         [[ `echo $__cpu_type__ | grep 'Nehemiah'` != '' ]]; then
      Processor=$CPU_ViaC3II
    elif [[ `echo $__cpu_type__ | grep 'C3'`   != '' ]] ||
         [[ `echo $__cpu_type__ | grep 'Ezra'` != '' ]]; then
      if [[ `echo $__cpu_type__ | grep ' 2 '` != '' ]]; then
        Processor=$CPU_ViaC3II
      else
        Processor=$CPU_ViaC3
      fi
    fi
  fi
}

DetectCPUModel()
{
  ##### Get informations from "/proc/cpuinfo"
  local __cpuinfo_vend__=`cat /proc/cpuinfo | grep 'vendor_id' | head -n1`
  local __cpuinfo_type__=`cat /proc/cpuinfo | grep 'model name' | head -n1`
  local __cpuinfo_flag__=`cat /proc/cpuinfo | grep 'flags' | head -n1`

  ##### Detect based on the vendor
  if [[ `echo $__cpuinfo_vend__ | grep 'Intel'` != '' ]]; then
    DetectCPUModel_Intel "$__cpuinfo_vend__" "$__cpuinfo_type__" "$__cpuinfo_flag__"
  elif [[ `echo $__cpuinfo_vend__ | grep 'AMD'` != '' ]]; then
    DetectCPUModel_AMD "$__cpuinfo_vend__" "$__cpuinfo_type__" "$__cpuinfo_flag__"
  else
    DetectCPUModel_Others "$__cpuinfo_vend__" "$__cpuinfo_type__" "$__cpuinfo_flag__"
  fi

  ##### Assume as any i586 compatible for unknown vendors and/or models
  if [[ $Processor == $CPU_Autodetect ]]; then
    Processor=$CPU_Anyi586Compatible
  fi

  ##### Print the detected model
  echo $_COLOR_PURPLE"Autodetecting processor type ... $_COLOR_BLUE$Processor$_COLOR_NORMAL"
  echo ''
}


#######################################################################
##### Functions for Creating Configured Files
#######################################################################

##### Generate sed instruction based on the user's choices
CreateSedScripts()
{
  #####
  ##### Determine the CPU model and extensions
  #####

  ##### Autodetect the processor type if asked
  if [[ $Processor == $CPU_Autodetect ]]; then
    DetectCPUModel
  fi

  ##### Determine postfix text for the selected CPU
  local __carc__;
    if [[ $Processor == $CPU_Anyi586Compatible ]]; then __carc__='i586'
  elif [[ $Processor == $CPU_Anyi686Compatible ]]; then __carc__='i686'
  elif [[ $Processor == $CPU_IntelPentium      ]]; then __carc__='pentium'
  elif [[ $Processor == $CPU_IntelPentiumMMX   ]]; then __carc__='pentium-mmx'
  elif [[ $Processor == $CPU_IntelPentiumPro   ]]; then __carc__='pentium-pro'
  elif [[ $Processor == $CPU_IntelPentiumII    ]]; then __carc__='pentium2'
  elif [[ $Processor == $CPU_IntelCeleron      ]]; then __carc__='pentium2'
  elif [[ $Processor == $CPU_IntelP2Xeon       ]]; then __carc__='pentium2'
  elif [[ $Processor == $CPU_IntelPentiumIII   ]]; then __carc__='pentium3'
  elif [[ $Processor == $CPU_IntelCeleron2     ]]; then __carc__='pentium3'
  elif [[ $Processor == $CPU_IntelP3Xeon       ]]; then __carc__='pentium3'
  elif [[ $Processor == $CPU_IntelPentiumM     ]]; then __carc__='pentium-m'
  elif [[ $Processor == $CPU_IntelCeleronM     ]]; then __carc__='pentium-m'
  elif [[ $Processor == $CPU_IntelPentium4     ]]; then __carc__='pentium4'
  elif [[ $Processor == $CPU_IntelCeleron3     ]]; then __carc__='pentium4'
  elif [[ $Processor == $CPU_IntelP4Xeon       ]]; then __carc__='pentium4'
  elif [[ $Processor == $CPU_IntelPrescott     ]]; then __carc__='prescott'
  elif [[ $Processor == $CPU_IntelCeleronD     ]]; then __carc__='prescott'
  elif [[ $Processor == $CPU_IntelNocona       ]]; then __carc__='nocona'
  elif [[ $Processor == $CPU_IntelPentiumD     ]]; then __carc__='nocona'
  elif [[ $Processor == $CPU_IntelCoreSolo     ]]; then __carc__='prescott'
  elif [[ $Processor == $CPU_IntelCoreDuo      ]]; then __carc__='prescott'
  elif [[ $Processor == $CPU_IntelCore2Duo     ]]; then __carc__='nocona'
  elif [[ $Processor == $CPU_AMDK6             ]]; then __carc__='k6'
  elif [[ $Processor == $CPU_AMDK6II           ]]; then __carc__='k6-2'
  elif [[ $Processor == $CPU_AMDK6III          ]]; then __carc__='k6-3'
  elif [[ $Processor == $CPU_AMDAthlon         ]]; then __carc__='athlon'
  elif [[ $Processor == $CPU_AMDAthlonTBird    ]]; then __carc__='athlon-tbird'
  elif [[ $Processor == $CPU_AMDDuron          ]]; then __carc__='athlon-tbird'
  elif [[ $Processor == $CPU_AMDAthlon4        ]]; then __carc__='athlon-4'
  elif [[ $Processor == $CPU_AMDAthlonXP       ]]; then __carc__='athlon-xp'
  elif [[ $Processor == $CPU_AMDAthlonMP       ]]; then __carc__='athlon-mp'
  elif [[ $Processor == $CPU_AMDK8             ]]; then __carc__='k8'
  elif [[ $Processor == $CPU_AMDOpteron        ]]; then __carc__='opteron'
  elif [[ $Processor == $CPU_AMDAthlon64       ]]; then __carc__='athlon64'
  elif [[ $Processor == $CPU_AMDAthlonSempron  ]]; then __carc__='athlon64'
  elif [[ $Processor == $CPU_AMDAthlonTurion   ]]; then __carc__='athlon64'
  elif [[ $Processor == $CPU_AMDAthlonFX       ]]; then __carc__='athlon-fx'
  elif [[ $Processor == $CPU_AMDAthlonX2       ]]; then __carc__='athlon64'
  elif [[ $Processor == $CPU_IDTWinChipC6      ]]; then __carc__='winchip-c6'
  elif [[ $Processor == $CPU_IDTWinChip2       ]]; then __carc__='winchip2'
  elif [[ $Processor == $CPU_ViaC3             ]]; then __carc__='c3'
  elif [[ $Processor == $CPU_ViaC3II           ]]; then __carc__='c3-2'
  else                                                  __carc__='i586'
  fi

  ##### Determine the number of the CPU's core(s)
  local __core__=1
  if [[ $Processor == $CPU_IntelCoreDuo  ]] ||
     [[ $Processor == $CPU_IntelCore2Duo ]] ||
     [[ $Processor == $CPU_AMDAthlonX2   ]]; then
    __core__=2
  fi

  ##### Determine the minimal (safest) CPU's extensions to be enabled
  local __mmx__='N'
  local __ss1__='N'
  local __ss2__='N'
  local __ss3__='N'
  local __3dn__='N'
    if [[ $Processor == $CPU_IntelPentiumMMX   ]]; then __mmx__='Y'
  elif [[ $Processor == $CPU_IntelPentiumII    ]]; then __mmx__='Y'
  elif [[ $Processor == $CPU_IntelCeleron      ]]; then __mmx__='Y'
  elif [[ $Processor == $CPU_IntelP2Xeon       ]]; then __mmx__='Y'
  elif [[ $Processor == $CPU_IntelPentiumIII   ]]; then __mmx__='Y'; __ss1__='Y'
  elif [[ $Processor == $CPU_IntelCeleron2     ]]; then __mmx__='Y'; __ss1__='Y'
  elif [[ $Processor == $CPU_IntelP3Xeon       ]]; then __mmx__='Y'; __ss1__='Y'
  elif [[ $Processor == $CPU_IntelPentiumM     ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_IntelCeleronM     ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_IntelPentium4     ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_IntelCeleron3     ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_IntelP4Xeon       ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_IntelPrescott     ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'; __ss3__='Y'
  elif [[ $Processor == $CPU_IntelCeleronD     ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'; __ss3__='Y'
  elif [[ $Processor == $CPU_IntelNocona       ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'; __ss3__='Y'
  elif [[ $Processor == $CPU_IntelPentiumD     ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'; __ss3__='Y'
  elif [[ $Processor == $CPU_IntelCoreSolo     ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'; __ss3__='Y'
  elif [[ $Processor == $CPU_IntelCoreDuo      ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'; __ss3__='Y'
  elif [[ $Processor == $CPU_IntelCore2Duo     ]]; then __mmx__='Y'; __ss1__='Y'; __ss2__='Y'; __ss3__='Y'
  elif [[ $Processor == $CPU_AMDK6             ]]; then __mmx__='Y'
  elif [[ $Processor == $CPU_AMDK6II           ]]; then __mmx__='Y'; __3dn__='Y'
  elif [[ $Processor == $CPU_AMDK6III          ]]; then __mmx__='Y'; __3dn__='Y'
  elif [[ $Processor == $CPU_AMDAthlon         ]]; then __mmx__='Y'; __3dn__='Y'
  elif [[ $Processor == $CPU_AMDAthlonTBird    ]]; then __mmx__='Y'; __3dn__='Y'
  elif [[ $Processor == $CPU_AMDDuron          ]]; then __mmx__='Y'; __3dn__='Y'
  elif [[ $Processor == $CPU_AMDAthlon4        ]]; then __mmx__='Y'; __3dn__='Y'; __ss1__='Y'
  elif [[ $Processor == $CPU_AMDAthlonXP       ]]; then __mmx__='Y'; __3dn__='Y'; __ss1__='Y'
  elif [[ $Processor == $CPU_AMDAthlonMP       ]]; then __mmx__='Y'; __3dn__='Y'; __ss1__='Y'
  elif [[ $Processor == $CPU_AMDK8             ]]; then __mmx__='Y'; __3dn__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_AMDOpteron        ]]; then __mmx__='Y'; __3dn__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_AMDAthlon64       ]]; then __mmx__='Y'; __3dn__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_AMDAthlonSempron  ]]; then __mmx__='Y'; __3dn__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_AMDAthlonTurion   ]]; then __mmx__='Y'; __3dn__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_AMDAthlonFX       ]]; then __mmx__='Y'; __3dn__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_AMDAthlonX2       ]]; then __mmx__='Y'; __3dn__='Y'; __ss1__='Y'; __ss2__='Y'
  elif [[ $Processor == $CPU_IDTWinChipC6      ]]; then __mmx__='Y'
  elif [[ $Processor == $CPU_IDTWinChip2       ]]; then __mmx__='Y'; __3dn__='Y'
  elif [[ $Processor == $CPU_ViaC3             ]]; then __mmx__='Y'; __3dn__='Y'
  elif [[ $Processor == $CPU_ViaC3II           ]]; then __mmx__='Y'; __ss1__='Y'
  fi

  ##### Validate the CPU extensions
  rm -f guess-ext
  eval $GNUGppCmd buildScript/cc/guess-ext.cc -o guess-ext >/dev/null 2>&1
  if (( $? != 0 )); then
    echo $_COLOR_RED"WARNING: Couldn't build the program needed for autodetecting the CPU extensions !"$_COLOR_NORMAL
  fi
  if [[ -x ./guess-ext ]]; then
    local __ext__=`./guess-ext`
    rm -f guess-ext
    if [[ $DetectPEx == 'Y' ]]; then
      echo $_COLOR_BROWN"NOTE: Using the autodetected CPU's MMX, 3DNow! and SSE extensions (overriding the default per CPU's settings)."$_COLOR_NORMAL
      __mmx__='N'; __ss1__='N'; __ss2__='N'; __ss3__='N'; __3dn__='N'
      if [[ $__ext__ == *_mmx_*   ]]; then __mmx__='Y'; fi
      if [[ $__ext__ == *_sse_*   ]]; then __ss1__='Y'; fi
      if [[ $__ext__ == *_sse2_*  ]]; then __ss2__='Y'; fi
      if [[ $__ext__ == *_sse3_*  ]]; then __ss3__='Y'; fi
      if [[ $__ext__ == *_3dnow_* ]]; then __3dn__='Y'; fi
    else
      if [[ $__mmx__ != 'Y' ]] && [[ $__ext__ == *_mmx_* ]]; then
        echo $_COLOR_BROWN"NOTE: The current CPU has MMX support but is not enabled by the default setting."$_COLOR_NORMAL
      elif [[ $__mmx__ == 'Y' ]] && [[ $__ext__ != *_mmx_* ]]; then
        echo $_COLOR_BROWN"NOTE: The current CPU has no MMX support but is enabled by the default setting."$_COLOR_NORMAL
      fi
      if [[ $__ss1__ != 'Y' ]] && [[ $__ext__ == *_sse_* ]]; then
        echo $_COLOR_BROWN"NOTE: The current CPU has SSE support but is not enabled by the default setting."$_COLOR_NORMAL
      elif [[ $__ss1__ == 'Y' ]] && [[ $__ext__ != *_sse_* ]]; then
        echo $_COLOR_BROWN"NOTE: The current CPU has no SSE support but is enabled by the default setting."$_COLOR_NORMAL
      fi
      if [[ $__ss2__ != 'Y' ]] && [[ $__ext__ == *_sse2_* ]]; then
        echo $_COLOR_BROWN"NOTE: The current CPU has SSE2 support but is not enabled by the default setting."$_COLOR_NORMAL
      elif [[ $__ss2__ == 'Y' ]] && [[ $__ext__ != *_sse2_* ]]; then
        echo $_COLOR_BROWN"NOTE: The current CPU has no SSE2 support but is enabled by the default setting."$_COLOR_NORMAL
      fi
      if [[ $__ss3__ != 'Y' ]] && [[ $__ext__ == *_sse3_* ]]; then
        echo $_COLOR_BROWN"NOTE: The current CPU has SSE3 support but is not enabled by the default setting."$_COLOR_NORMAL
      elif [[ $__ss3__ == 'Y' ]] && [[ $__ext__ != *_sse3_* ]]; then
        echo $_COLOR_BROWN"NOTE: The current CPU has no SSE3 support but is enabled by the default setting."$_COLOR_NORMAL
      fi
      if [[ $__3dn__ != 'Y' ]] && [[ $__ext__ == *_3dnow_* ]]; then
        echo $_COLOR_BROWN"NOTE: The current CPU has 3DNow! support but is not enabled by the default setting."$_COLOR_NORMAL
      elif [[ $__3dn__ == 'Y' ]] && [[ $__ext__ != *_3dnow_* ]]; then
        echo $_COLOR_BROWN"NOTE: The current CPU has no 3DNow! support but is enabled by the default setting."$_COLOR_NORMAL
      fi
    fi
  elif [[ $DetectPEx == 'Y' ]]; then
    echo $_COLOR_RED"WARNING: Couldn't autodetect the current CPU's MMX, 3DNow! and SSE extensions, using the default per CPU's settings."$_COLOR_NORMAL
  fi


  #####
  ##### Generate sed instruction for modifying makefile configuration and spec files
  #####

  echo $_COLOR_CYAN"Creating sed scripts for use in configuration ..."$_COLOR_NORMAL

  ##### File notes
  echo "s/$_STR_MAKE_N1_ORG/$_STR_MAKE_N1_REP/" > $SedScrMake
  echo "/$_STR_MAKE_N2_ORG/d" >> $SedScrMake
  echo "s/$_STR_SPEC_I_CHG0/$_STR_SPEC_I_REP0/" > $SedScrMake
  echo "/$_STR_SPEC_I_DEL0/d" >> $SedScrMake
  echo "/$_STR_SPEC_I_DEL1/d" >> $SedScrMake
  echo "/$_STR_SPEC_I_DEL2/d" >> $SedScrMake
  echo "/$_STR_SPEC_I_DEL3/d" >> $SedScrMake
  echo "/$_STR_SPEC_I_DEL4/d" >> $SedScrMake
  echo "/$_STR_SPEC_I_DEL5/d" >> $SedScrMake
  echo "/$_STR_SPEC_I_DEL6/d" >> $SedScrMake
  echo "/$_STR_SPEC_I_DEL7/d" >> $SedScrMake
  echo "/$_STR_SPEC_I_DEL8/d" >> $SedScrMake

  ##### Build root directory
  local __brd__=`pwd`
  echo "s/$_STR_SPEC_I_VGBR/${__brd__//\//\/}/" >> $SedScrMake

  ##### VortexGE version
  echo "s/$_STR_MAKE_I_SVER/$SVER/" >> $SedScrMake
  echo "s/$_STR_MAKE_I_FVER/$FVER/" >> $SedScrMake
  echo "s/$_STR_MAKE_I_RVER/$RVER/" >> $SedScrMake
  echo "s/$_STR_MAKE_I_LVER/$LVER/" >> $SedScrMake

  ##### VortexGE installation directories
  local __inc__=$VGInsDSel"include/vge"
  local __lib__=$VGInsDSel"lib"
  local __exe__=$VGInsDSel"bin"
  local __doc__=$VGInsDSel"share/doc/vge"
  echo "s/$_STR_MAKE_I_VGII/${__inc__//\//\/}/"   >> $SedScrMake
  echo "s/$_STR_MAKE_I_VGIL/${__lib__//\//\/}/"   >> $SedScrMake
  echo "s/$_STR_MAKE_I_VGIE/${__exe__//\//\/}/"   >> $SedScrMake
  echo "s/$_STR_MAKE_I_VGID/${__doc__//\//\/}/"   >> $SedScrMake
  echo "s/$_STR_SPEC_I_RPMB/${VGRPMBldD//\//\/}/" >> $SedScrMake

  ##### Shall the API documentions be built ?
  if [[ $BuildAPID == 'Y' ]]; then
    echo "s/$_STR_MAKE_I_APID/1/" >> $SedScrMake
  else
    echo "s/$_STR_MAKE_I_APID/0/" >> $SedScrMake
  fi

  ##### Debugging information and GCH
  if [[ $UseDebugI == 'Y' ]]; then
    echo "s/$_STR_MAKE_I_UGCH/0/" >> $SedScrMake
    echo "s/$_STR_MAKE_I_UDBG/-g/" >> $SedScrMake
  else
    echo "s/$_STR_MAKE_I_UGCH/1/" >> $SedScrMake
    echo "s/$_STR_MAKE_I_UDBG//" >> $SedScrMake
  fi

  ##### Select the type of library to be built
  if [[ $BuildALib == 'Y' ]]; then
    echo "s/$_STR_MAKE_I_BLDA/1/" >> $SedScrMake
  else
    echo "s/$_STR_MAKE_I_BLDA/0/" >> $SedScrMake
  fi
  echo "s/$_STR_MAKE_I_BLDS/1/" >> $SedScrMake

  ##### Enable Position Independent Code (PIC) build except for i386 hardware platform
  if [[ `uname -m` == i*86 ]]; then
    echo "s/$_STR_MAKE_I_OPSO//" >> $SedScrMake
  else
    echo "s/$_STR_MAKE_I_OPSO/-fPIC -D__VGE_PIC_BUILD__/" >> $SedScrMake
  fi

  ##### X11 directories & files
   FixDir $X11IncLoc X11IncLoc
  FixDir $X11LibLoc X11LibLoc
  echo "s/$_STR_MAKE_I_X11I/-isystem${X11IncLoc//\//\/}/" >> $SedScrMake
  echo "s/$_STR_MAKE_I_X11L/${X11LibLoc//\//\/}/" >> $SedScrMake
  if [[ $Xxf86vmso == 'Y' ]]; then
    echo "s/$_STR_MAKE_H_86VM/1/" >> $SedScrMake
  else
    echo "s/$_STR_MAKE_H_86VM/0/" >> $SedScrMake
  fi

  ##### Additional search directories
  if [[ $AlsIncLoc != '' ]] && [[ $AlsLibLoc != '' ]]; then
    AddCustomIncludeLocation $AlsIncLoc
    AddCustomLibraryLocation $AlsLibLoc
  fi
  if [[ $OssIncLoc != '' ]]; then
    AddCustomIncludeLocation $OssIncLoc
  fi
  if [[ $UseLibPng == 'Y' ]] && [[ $PngIncLoc != '' ]] && [[ $PngLibLoc != '' ]]; then
    AddCustomIncludeLocation $PngIncLoc
    AddCustomLibraryLocation $PngLibLoc
  fi
  if [[ $UseLibJpg == 'Y' ]] && [[ $JpgIncLoc != '' ]] && [[ $JpgLibLoc != '' ]]; then
    AddCustomIncludeLocation $JpgIncLoc
    AddCustomLibraryLocation $JpgLibLoc
  fi
  if [[ $UseLibTif == 'Y' ]] && [[ $TifIncLoc != '' ]] && [[ $TifLibLoc != '' ]]; then
    AddCustomIncludeLocation $TifIncLoc
    AddCustomLibraryLocation $TifLibLoc
  fi
  if [[ $UseLibGif == 'Y' ]] && [[ $GifIncLoc != '' ]] && [[ $GifLibLoc != '' ]]; then
    AddCustomIncludeLocation $GifIncLoc
    AddCustomLibraryLocation $GifLibLoc
  fi
  if [[ $AddIncLoc != '' ]]; then
    echo "s/$_STR_MAKE_I_ADDI/${AddIncLoc//\//\/}/" >> $SedScrMake
  else
    echo "s/$_STR_MAKE_I_ADDI//" >> $SedScrMake
  fi
  if [[ $AddLibLoc != '' ]]; then
    echo "s/$_STR_MAKE_I_ADDL/${AddLibLoc//\//\/}/" >> $SedScrMake
  else
    echo "s/$_STR_MAKE_I_ADDL//" >> $SedScrMake
  fi

  ##### User libraries : X11 extensions
  local __lxs__=' '
  if [[ $UseXF86VM == 'Y' ]] && [[ $Xxf86vmso == 'Y' ]]; then
    __lxs__=$__lxs__'-lXxf86vm '
  fi
  if [[ $UseXShmEx == 'Y' ]]; then
    __lxs__=$__lxs__'-lXext '
  fi
  if [[ $UseXMuExt == 'Y' ]]; then
    __lxs__=$__lxs__'-lXmu '
  fi

  ##### User libraries : others
  local __lps__=''
  if [[ $AlsIncLoc != '' ]] && [[ $AlsLibLoc != '' ]]; then
    __lps__=$__lps__'-lasound '
  fi
  if [[ $UseLibPng == 'Y' ]] && [[ $PngIncLoc != '' ]] && [[ $PngLibLoc != '' ]]; then
    __lps__=$__lps__'-lpng '
  fi
  if [[ $UseLibJpg == 'Y' ]] && [[ $JpgIncLoc != '' ]] && [[ $JpgLibLoc != '' ]]; then
    __lps__=$__lps__'-ljpeg '
  fi
  if [[ $UseLibTif == 'Y' ]] && [[ $TifIncLoc != '' ]] && [[ $TifLibLoc != '' ]]; then
    __lps__=$__lps__'-ltiff '
  fi
  if [[ $UseLibGif == 'Y' ]] && [[ $GifIncLoc != '' ]] && [[ $GifLibLoc != '' ]]; then
    __lps__=$__lps__$GifLibUse' '
  fi

  ##### Put the user libraries in the makefile configuration file
  echo "s/$_STR_MAKE_I_ULIB/"$__lps__"-lz -lX11"$__lxs__"-lXpm/" >> $SedScrMake

  ##### RPM build target
  echo "s/$_STR_SPEC_I_RPMT/i386/" >> $SedScrMake
  echo "s/$_STR_SPEC_I_RPMC/$__carc__/" >> $SedScrMake

  ##### Adjust properties based on the gcc version
  echo "s/$_STR_MAKE_I_GNUC/${GNUGccCmd//\//\\/}/" >> $SedScrMake
  echo "s/$_STR_MAKE_I_GNUP/${GNUGppCmd//\//\\/}/" >> $SedScrMake
  if (( GNUGppVer >= 40000 )); then
    echo "s/$_STR_MAKE_I_AGCO/-fvisibility=hidden/" >> $SedScrMake
    echo "s/$_STR_MAKE_I_AGPO/-fvisibility=hidden -fvisibility-inlines-hidden/" >> $SedScrMake
  else
    echo "/$_STR_MAKE_I_AGCO/d" >> $SedScrMake
    echo "/$_STR_MAKE_I_AGPO/d" >> $SedScrMake
  fi

  ##### Optimization level
  echo "s/$_STR_MAKE_I_OLVL/-O$OptzLevel/" >> $SedScrMake

  ##### CPU model and extensions
  local __flg__
  if [[ $__mmx__ == 'Y' ]]; then
    __flg__='-mmmx'
  else
    __flg__='-mno-mmx'
  fi
  if [[ $__ss1__ == 'Y' ]]; then
    __flg__="$__flg__ -msse"
  else
    __flg__="$__flg__ -mno-sse"
  fi
  if [[ $__ss2__ == 'Y' ]]; then
    __flg__="$__flg__ -msse2"
  else
    __flg__="$__flg__ -mno-sse2"
  fi
  if [[ $__ss3__ == 'Y' ]]; then
    __flg__="$__flg__ -msse3"
  else
    __flg__="$__flg__ -mno-sse3"
  fi
  if [[ $__3dn__ == 'Y' ]]; then
    __flg__="$__flg__ -m3dnow"
  else
    __flg__="$__flg__ -mno-3dnow"
  fi
  if [[ $__ss1__ == 'Y' ]] && [[ $__ss2__ == 'Y' ]]; then
    echo "s/$_STR_MAKE_I_FPMT/-mfpmath=sse/" >> $SedScrMake
  else
    echo "s/$_STR_MAKE_I_FPMT/-mfpmath=387/" >> $SedScrMake
  fi
  echo "s/$_STR_MAKE_I_PEXT/$__flg__/" >> $SedScrMake
  echo "s/$_STR_MAKE_I_ARCH/-march=$__carc__/" >> $SedScrMake

  ##### Enable stripping only if debugging information is not enabled
  if [[ $UseDebugI == 'Y' ]]; then
    echo "s/$_STR_MAKE_I_STRP/echo > \/dev\/null/" >> $SedScrMake
  else
    echo "s/$_STR_MAKE_I_STRP/strip -R .comment --strip-unneeded \$@/" >> $SedScrMake
  fi

  ##### Use usleep for idling if it is available, otherwise use sleep
  if [[ -x `which usleep 2>&1` ]]; then
    echo "s/$_STR_MAKE_I_IDLE/usleep 100000/" >> $SedScrMake
  else
    echo "s/$_STR_MAKE_I_IDLE/sleep 1/" >> $SedScrMake
  fi


  #####
  ##### Generate sed instruction for modifying configuration header file
  #####

  ##### File notes
  echo "s/VConfig.h.in/VConfig.h/" > $SedScrVcth
  echo "s/$_STR_HDRF_N1_ORG/$_STR_HDRF_N1_REP/" >> $SedScrVcth
  echo "/$_STR_HDRF_N2_ORG/d" >> $SedScrVcth

  ##### Path to where "libVGE.messages.cat" catalog directory will be installed
  echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
  echo "#define __LIB_HC_MCTLG_PATH__ \""$VGInsDSel"lib/libVGE.messages.cat\"" >> $SedScrVcth

  ##### Number of CPU's core
  echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
  echo "#define __CPU_CORES__ $__core__" >> $SedScrVcth

  ##### Colored messages ?
  if [[ $ColorMsgs == 'Y' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_COLORED_MESSAGES__" >> $SedScrVcth
  fi

  ##### Video system ?
  echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
  echo "#define __USE_X11_VIDEO__" >> $SedScrVcth
  if [[ $UseXF86VM == 'Y' ]] && [[ $Xxf86vmso == 'Y' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_XF86VM__" >> $SedScrVcth
  fi
  if [[ $UseXShmEx == 'Y' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_XSHM__" >> $SedScrVcth
  fi
  if [[ $UseXMuExt == 'Y' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_XMU__" >> $SedScrVcth
  fi
  echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
  echo "#define __USE_DITHERING_FOR_8BPP__" >> $SedScrVcth

  ##### Audio system ?
  if [[ $OssIncLoc != '' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_OSS_AUDIO__" >> $SedScrVcth
  fi

  if [[ $AlsIncLoc != '' ]] && [[ $AlsLibLoc != '' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_ALSA_AUDIO__" >> $SedScrVcth
  fi

  ##### MIDI system ?
  echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
  echo "#define __USE_TIM_MIDI__" >> $SedScrVcth

  #### libpng
  if [[ $UseLibPng == 'Y' ]] && [[ $PngIncLoc != '' ]] && [[ $PngLibLoc != '' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_LIBPNG__" >> $SedScrVcth
  fi

  #### libjpeg
  if [[ $UseLibJpg == 'Y' ]] && [[ $JpgIncLoc != '' ]] && [[ $JpgLibLoc != '' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_LIBJPEG__" >> $SedScrVcth
  fi

  #### libtiff
  if [[ $UseLibTif == 'Y' ]] && [[ $TifIncLoc != '' ]] && [[ $TifLibLoc != '' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_LIBTIFF__" >> $SedScrVcth
  fi

  #### libgif
  if [[ $UseLibGif == 'Y' ]] && [[ $GifIncLoc != '' ]] && [[ $GifLibLoc != '' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_LIBUNGIF__" >> $SedScrVcth
  fi

  ##### Inline assembly optimizations ?
  if [[ $InlineAsm == 'Y' ]]; then
    echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
    echo "#define __USE_ASM__" >> $SedScrVcth
    ##### Generate build macros instruction based on the settings
    ##### from the "CPU specific optimizations" section above
    local __amo__='N'
    if [[ $__mmx__ == 'Y' ]]; then
      echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
      echo "#define __USE_MMX__" >> $SedScrVcth
    fi
    if [[ $__ss1__ == 'Y' ]]; then
      __amo__=$AlignedMO
      echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
      echo "#define __USE_SSE__" >> $SedScrVcth
    fi
    if [[ $__ss2__ == 'Y' ]]; then
      __amo__=$AlignedMO
      echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
      echo "#define __USE_SSE2__" >> $SedScrVcth
    fi
    if [[ $__ss3__ == 'Y' ]]; then
      __amo__=$AlignedMO
      echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
      echo "#define __USE_SSE3__" >> $SedScrVcth
    fi
    if [[ $__3dn__ == 'Y' ]]; then
      echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
      echo "#define __USE_3DNOW__" >> $SedScrVcth
    fi
    if [[ $__amo__ == 'Y' ]]; then
      echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
      echo "#define __USE_ALIGNED_MEMOPS__" >> $SedScrVcth
    fi
  else
    if [[ $QuadWrdMO == 'Y' ]]; then
      echo "/$_STR_HDRF_I_BMCR/a\\" >> $SedScrVcth
      echo "#define __USE_QW_MEMOPS__" >> $SedScrVcth
    fi
  fi

  ##### Delete the sign text
  echo "/$_STR_HDRF_I_BMCR/d" >> $SedScrVcth
}

##### Delete the temporary sed scripts
DeleteSedScripts()
{
  echo $_COLOR_CYAN"Deleting sed scripts used in configuration ..."$_COLOR_NORMAL
  rm -f $SedScrMake
  rm -f $SedScrVcth
}

##### Delete all existing symbolic links to makefiles
DeleteSymLinks()
{
  echo $_COLOR_CYAN"Deleting symbolic links to makefiles ..."$_COLOR_NORMAL
  rm -f Makefile
  rm -f Makefile.install
  rm -f lib/Makefile
  rm -f lib/2DManipulation/Makefile
  rm -f lib/2DManipulation/Fractal/Makefile
  rm -f lib/3DEnvironment/Makefile
  rm -f lib/AudioEnvironment/Makefile
  rm -f lib/AudioEnvironment/PCM_Dummy/Makefile
  rm -f lib/AudioEnvironment/PCM_OSS/Makefile
  rm -f lib/AudioEnvironment/PCM_ALSA/Makefile
  rm -f lib/AudioEnvironment/MID_Dummy/Makefile
  rm -f lib/AudioEnvironment/MID_libTiMidity/Makefile
  rm -f lib/Base/Makefile
  rm -f lib/GameObject/Makefile
  rm -f lib/HighLevelClass/Makefile
  rm -f lib/System/Makefile
  rm -f lib/VideoEnvironment/Makefile
  rm -f lib/VideoEnvironment/VID_X11/Makefile
  rm -f lib/VortexLua/Makefile
  rm -f demo/Makefile
  rm -f util/Makefile
  rm -f rawBenchmark/Makefile
  rm -f doc/tutrefs/cc/Makefile

  rm -f 3rdParty/libTiMidity/Makefile
  rm -f 3rdParty/lua/Makefile
  rm -f 3rdParty/xf86vm/Makefile

  rm -f tutorial

}

##### Create symbolic links to makefiles
CreateSymLinks()
{
  echo $_COLOR_CYAN"Creating symbolic links to makefiles ..."$_COLOR_NORMAL
    ln -sf          buildScript/Makefile.main    Makefile
    ln -sf          buildScript/Makefile.install Makefile.install
    ln -sf       ../buildScript/Makefile.vglib   lib/Makefile
    ln -sf    ../../buildScript/Makefile.cmpl    lib/2DManipulation/Makefile
    ln -sf ../../../buildScript/Makefile.cmpl    lib/2DManipulation/Fractal/Makefile
    ln -sf    ../../buildScript/Makefile.cmpl    lib/3DEnvironment/Makefile
    ln -sf    ../../buildScript/Makefile.cmpl    lib/AudioEnvironment/Makefile
    ln -sf ../../../buildScript/Makefile.cmpl    lib/AudioEnvironment/PCM_Dummy/Makefile
  if [[ $OssIncLoc != '' ]]; then
    ln -sf ../../../buildScript/Makefile.cmpl    lib/AudioEnvironment/PCM_OSS/Makefile
  fi
  if [[ $AlsIncLoc != '' ]] &&
     [[ $AlsLibLoc != '' ]]; then
    ln -sf ../../../buildScript/Makefile.cmpl    lib/AudioEnvironment/PCM_ALSA/Makefile
  fi
    ln -sf ../../../buildScript/Makefile.cmpl    lib/AudioEnvironment/MID_Dummy/Makefile
    ln -sf ../../../buildScript/Makefile.cmpl    lib/AudioEnvironment/MID_libTiMidity/Makefile
    ln -sf    ../../buildScript/Makefile.cmpl    lib/Base/Makefile
    ln -sf    ../../buildScript/Makefile.cmpl    lib/GameObject/Makefile
    ln -sf    ../../buildScript/Makefile.cmpl    lib/HighLevelClass/Makefile
    ln -sf    ../../buildScript/Makefile.cmpl    lib/System/Makefile
    ln -sf    ../../buildScript/Makefile.cmpl    lib/VideoEnvironment/Makefile
    ln -sf ../../../buildScript/Makefile.vidx11  lib/VideoEnvironment/VID_X11/Makefile
    ln -sf    ../../buildScript/Makefile.cmpl    lib/VortexLua/Makefile
    ln -sf       ../buildScript/Makefile.demo    demo/Makefile
    ln -sf       ../buildScript/Makefile.util    util/Makefile
    ln -sf       ../buildScript/Makefile.bench   rawBenchmark/Makefile
    ln -sf ../../../buildScript/Makefile.tutor   doc/tutrefs/cc/Makefile

  ln -sf ../buildScript/Makefile.libtimidity 3rdParty/libTiMidity/Makefile
  ln -sf ../buildScript/Makefile.lua         3rdParty/lua/Makefile
  ln -sf ../buildScript/Makefile.xf86vm      3rdParty/xf86vm/Makefile

  ln -sf doc/tutrefs/cc tutorial
}

##### Delete makefile configuration file
DeleteConfigMakefile()
{
  echo $_COLOR_CYAN"Deleting makefile configuration file ..."$_COLOR_NORMAL
  rm -f buildScript/Config
}

##### Create makefile configuration file
CreateConfigMakefile()
{
  echo $_COLOR_CYAN"Creating makefile configuration file ..."$_COLOR_NORMAL
  $GNUSedCmd -f $SedScrMake 'buildScript/in/Config.in' > 'buildScript/Config'
}

##### Delete other configured files
DeleteOCFiles()
{
  echo $_COLOR_CYAN"Deleting other configured files ..."$_COLOR_NORMAL

  rm -f lib/VConfig.h
  rm -f util/vge-config

  rm -f buildScript/vge-autodeps
  rm -f buildScript/Doxyfile
  rm -f buildScript/VortexGE.spec
}

##### Create other configured files
CreateOCFiles()
{
  echo $_COLOR_CYAN"Creating other configured files ..."$_COLOR_NORMAL

  $GNUSedCmd -f $SedScrVcth 'lib/VConfig.h.in'      > 'lib/VConfig.h'
  $GNUSedCmd -f $SedScrMake 'util/in/vge-config.in' > 'util/vge-config'

  $GNUSedCmd -f $SedScrMake 'buildScript/in/vge-autodeps.in'  > 'buildScript/vge-autodeps'
  $GNUSedCmd -f $SedScrMake 'buildScript/in/Doxyfile.in'      > 'buildScript/Doxyfile'

  chmod u+x util/vge-config
  chmod u+x buildScript/vge-autodeps

  ##### Generate the spec file
  $GNUSedCmd -f $SedScrMake 'buildScript/in/VortexGE.spec.in.0.prefix'    > 'buildScript/VortexGE.spec'
  $GNUSedCmd -f $SedScrMake 'buildScript/in/VortexGE.spec.in.1.binary'   >> 'buildScript/VortexGE.spec'
  $GNUSedCmd -f $SedScrMake 'buildScript/in/VortexGE.spec.in.2.devel'    >> 'buildScript/VortexGE.spec'
  if [[ $BuildALib == 'Y' ]]; then
    $GNUSedCmd -f $SedScrMake 'buildScript/in/VortexGE.spec.in.3.static' >> 'buildScript/VortexGE.spec'
  fi
  if [[ $BuildAPID == 'Y' ]]; then
    $GNUSedCmd -f $SedScrMake 'buildScript/in/VortexGE.spec.in.4.doc'    >> 'buildScript/VortexGE.spec'
  fi
  $GNUSedCmd -f $SedScrMake 'buildScript/in/VortexGE.spec.in.5.postfix'  >> 'buildScript/VortexGE.spec'
}

##### Delete directories
DeleteDirectories()
{
  echo $_COLOR_CYAN"Deleting directories ..."$_COLOR_NORMAL
  rm -rf .mediaCaches
  rm -rf doc/api
  rm -rf $VGRPMBldD
}

##### Create directories
CreateDirectories()
{
  echo $_COLOR_CYAN"Creating directories ..."$_COLOR_NORMAL

  mkdir .mediaCaches
  chmod u=rwx,g=rx,o=rx .mediaCaches

  mkdir doc/api
  chmod u=rwx,g=rx,o=rx doc/api

  mkdir $VGRPMBldD
  chmod u=rwx,g=rx,o=rx $VGRPMBldD
}

##### Delete miscellaneous stuffs
DeleteMiscStuffs()
{
  echo $_COLOR_CYAN"Deleting miscellaneous stuffs ..."$_COLOR_NORMAL
  rm -f $GppGVTestF.cc $GppGVTestF
}

##### Perform cleaning
Clean()
{
  DeleteSymLinks
  DeleteConfigMakefile
  DeleteOCFiles
  DeleteSedScripts
  DeleteDirectories
  DeleteMiscStuffs
  echo ''
  ShowCleanComplete
  ExitScript 0
}
