/**
 * \file FixedPointConsts.h
 * \internal
*/
// ############################################################################################
//  FixedPointConsts.h
//  Definition of fixed point constants used when rendering
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _3D_ENVIRONMENT___FIXED_POINT_CONSTS_H_
#define _3D_ENVIRONMENT___FIXED_POINT_CONSTS_H_

// VortexGE headers
#include "../Base/VBasics.h"


namespace vge {

  // Fixed point constants used when rendering
  // !!! DO NOT CHANGE THEM !!!
  const uint32_t _ZSTEP_PREC_ = uint32_t(24);                               // Z-buffer precision
  const uint32_t _XSTEP_PREC_ = uint32_t(10);                               // Horizontal pixel scan precision
  const uint32_t _XCEIL_FRAC_ = uint32_t((0x00000001 << _XSTEP_PREC_) - 1); // Horizontal pixel scan ceiling fractional
  const uint32_t _ISTEP_PREC_ = uint32_t(14);                               // Intensity scan (light & fog) precision
  const uint32_t _TSTEP_PREC_ = uint32_t(14);                               // Texture scan (U & V) precision
  const uint32_t _TFRAC_MASK_ = uint32_t(0xFFFFFFFF << _TSTEP_PREC_);       // Texture fractional (garbage) removal mask
  const uint32_t _SSTEP_PREC_ = uint32_t(_TSTEP_PREC_);                     // Image (2D sprite) scan precision
  const uint32_t _SDDIV_FACT_ = uint32_t(5);                                // Subdivision shift (division) factor

 } // namespace vge
#endif
