// ############################################################################################
//  FoggedHLineClippingFunctions.cc
//  Implementation of fogged horizontal line segment clipping functions
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  Some of the source codes in this file were written based on the book :
//    Cutting-Edge 3D Game Programming with C++
//    John De Goes
//    Coriolis Group, Inc.
//    USA, 1996
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include "Panel3D.h"


namespace vge {

// Fogged, untextured, flat shaded
void _FogUF_TGClipHLSeg_(int32_t &n32X1, int32_t &n32X2, int32_t &n32Z, int32_t &n32ZStep,
                         int32_t &n32F, int32_t &n32FStep)
{
  if(n32X1 < C3DEnv::Active3DEnv()->_n32MinX) {
    n32Z += (n32ZStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32F += (n32FStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32X1 = C3DEnv::Active3DEnv()->_n32MinX;
  }
  else if(n32X1 > C3DEnv::Active3DEnv()->_n32MaxX)
    n32X1 = C3DEnv::Active3DEnv()->_n32MaxX;

  if(n32X2 < C3DEnv::Active3DEnv()->_n32MinX)
    n32X2 = C3DEnv::Active3DEnv()->_n32MinX;
  else if(n32X2 > C3DEnv::Active3DEnv()->_n32MaxX)
    n32X2 = C3DEnv::Active3DEnv()->_n32MaxX;
}

// Fogged, untextured, gouraud shaded
void _FogUG_TGClipHLSeg_(int32_t &n32X1, int32_t &n32X2, int32_t &n32Z, int32_t &n32ZStep,
                         int32_t &n32R, int32_t &n32RStep, int32_t &n32G, int32_t &n32GStep,
                         int32_t &n32B, int32_t &n32BStep, int32_t &n32F, int32_t &n32FStep)
{
  if(n32X1 < C3DEnv::Active3DEnv()->_n32MinX) {
    n32Z += (n32ZStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32R += (n32RStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32G += (n32GStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32B += (n32BStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32F += (n32FStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32X1 = C3DEnv::Active3DEnv()->_n32MinX;
  }
  else if(n32X1 > C3DEnv::Active3DEnv()->_n32MaxX)
    n32X1 = C3DEnv::Active3DEnv()->_n32MaxX;

  if(n32X2 < C3DEnv::Active3DEnv()->_n32MinX)
    n32X2 = C3DEnv::Active3DEnv()->_n32MinX;
  else if(n32X2 > C3DEnv::Active3DEnv()->_n32MaxX)
    n32X2 = C3DEnv::Active3DEnv()->_n32MaxX;
}

// Fogged, textured, flat shaded
void _FogTF_TGClipHLSeg_(int32_t &n32X1, int32_t &n32X2, int32_t &n32Z, int32_t &n32ZStep,
                         int32_t &n32U, int32_t &n32UStep, int32_t &n32V, int32_t &n32VStep,
                         int32_t &n32F, int32_t &n32FStep)
{
  if(n32X1 < C3DEnv::Active3DEnv()->_n32MinX) {
    n32Z += (n32ZStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32U += (n32UStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32V += (n32VStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32F += (n32FStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32X1 = C3DEnv::Active3DEnv()->_n32MinX;
  }
  else if(n32X1 > C3DEnv::Active3DEnv()->_n32MaxX)
    n32X1 = C3DEnv::Active3DEnv()->_n32MaxX;

  if(n32X2 < C3DEnv::Active3DEnv()->_n32MinX)
    n32X2 = C3DEnv::Active3DEnv()->_n32MinX;
  else if(n32X2 > C3DEnv::Active3DEnv()->_n32MaxX)
    n32X2 = C3DEnv::Active3DEnv()->_n32MaxX;
}

// Fogged, textured, gouraud shaded
void _FogTG_TGClipHLSeg_(int32_t &n32X1, int32_t &n32X2, int32_t &n32Z, int32_t &n32ZStep,
                         int32_t &n32U, int32_t &n32UStep, int32_t &n32V, int32_t &n32VStep,
                         int32_t &n32R, int32_t &n32RStep, int32_t &n32G, int32_t &n32GStep,
                         int32_t &n32B, int32_t &n32BStep, int32_t &n32F, int32_t &n32FStep)
{
  if(n32X1 < C3DEnv::Active3DEnv()->_n32MinX) {
    n32Z += (n32ZStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32R += (n32RStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32G += (n32GStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32B += (n32BStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32U += (n32UStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32V += (n32VStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32F += (n32FStep * (C3DEnv::Active3DEnv()->_n32MinX - n32X1));
    n32X1 = C3DEnv::Active3DEnv()->_n32MinX;
  }
  else if(n32X1 > C3DEnv::Active3DEnv()->_n32MaxX)
    n32X1 = C3DEnv::Active3DEnv()->_n32MaxX;

  if(n32X2 < C3DEnv::Active3DEnv()->_n32MinX)
    n32X2 = C3DEnv::Active3DEnv()->_n32MinX;
  else if(n32X2 > C3DEnv::Active3DEnv()->_n32MaxX)
    n32X2 = C3DEnv::Active3DEnv()->_n32MaxX;
}

} // namespace vge

