/** \file NonDummyAudioEnvBaseClass.h */
// ############################################################################################
//  NonDummySoundBufferBaseClass.h
//  Definition of CAudioEnvCImp class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _AUDIO_ENVIRONMENT___NON_DUMMY_AUDIO_ENV_BASE_CLASS_H_
#define _AUDIO_ENVIRONMENT___NON_DUMMY_AUDIO_ENV_BASE_CLASS_H_

// VortexGE headers
#include "../System/Thread.h"
#include "AudioEnvironment.h"


namespace vge {

  // Foward reference from "lib/AudioEnvironment/NonDummySoundBufferBaseClass.h"
  class CSoundBufferCImp;

  // Maximum sound buffers can be played together
  const uint32_t _MAX_SB_ = uint32_t(16);


  //!
  //! \internal
  //! \brief Common DSP data
  //!
  struct _TCommonDSPData {
    _TAudioFormat tFormat;       // The audio data format
    uint32_t      un32FragSize;  // The fragment size in bytes
    uint32_t      un32FragSize2; // The fragment size in bytes / 2
    uint32_t      un32FragSize4; // The fragment size in bytes / 4
    uint32_t      un32WriteSize; // The primary sound buffer write size
    u_char        un32WriteIncF; // The primary sound buffer write increment factor
    uint32_t      un32PBuffSize; // The primary sound buffer size in bytes
    u_char       *pucPBuff;      // The primary sound buffer
    u_char       *pucPBuffMax;   // The primary sound buffer maximum address
    u_char       *pucPBuffRead;  // The primary sound buffer read address
    u_char       *pucPBuffWrite; // The primary sound buffer write address
    uint8_t       un8ToWrite;    // Total number of blocks to be written
  };


  //!
  //! \brief Non-dummy audio environment common class
  //!
  //! \api pretty stable
  class VGE_EXPORT CAudioEnvCImp : public CAudioEnv {
    public:
      CAudioEnvCImp();
      virtual ~CAudioEnvCImp() throw();

      // Friend classes
      friend class CSoundBufferCImp;

    protected:
      // Sound mixing functions
      static void _DSPMixerU8m();
      static void _DSPMixerU8s();
      static void _DSPMixerS8m();
      static void _DSPMixerS8s();
      static void _DSPMixerU16m();
      static void _DSPMixerU16s();
      static void _DSPMixerS16m();
      static void _DSPMixerS16s();
      static void _IDSPMixerU16m();
      static void _IDSPMixerU16s();
      static void _IDSPMixerS16m();
      static void _IDSPMixerS16s();

      // Data
      CThread             _csTWriter;     // Audio device writer thread
      CMutex              _csMutex;       // Synchronization mutex
      _TCommonDSPData    *_tCD;           // Pointer to common DSP data structure
      ESoundBufferFormat  _eFormat;       // Audio data format to be returned by GetAudioSpec()
      CMidiPlayer        *_pcsMidiPlayer; // Pointer to the MIDI player instance
      void (*_pfnDoSoundMixing)();        // Pointer to the sound mixing function

      struct _TPlayedSBIData {
        _TPlayedSBData    tData; // Played sound buffer data
        CSoundBufferCImp *pcsSB; // Sound buffer instance
      } _atPSB[_MAX_SB_];        // List of played sound buffer

      // Pointer to the one and the only audio environment instance
      static CAudioEnvCImp *_instAudioEnv;
  };

} // namespace vge
#endif
