/** \file ADMYSoundBuffer.h */
// ############################################################################################
//  ADMYSoundBuffer.h
//  Definition & implementation of CSoundBuffer class using dummy (NULL) audio system
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _AUDIO_ENVIRONMENT___PCM_DUMMY___ADMY_SOUND_BUFFER_H_
#define _AUDIO_ENVIRONMENT___PCM_DUMMY___ADMY_SOUND_BUFFER_H_

// VortexGE headers
#include "../SoundBuffer.h"


namespace vge {

  //!
  //! \brief Dummy audio sound-buffer class
  //!
  //! \api pretty stable
  class VGE_EXPORT CDummySoundBuffer : public CSoundBuffer {
    public:
      virtual ~CDummySoundBuffer() throw();
      // Playback functions
      virtual void Play(bool bLooping = false);
      virtual bool IsPlaying() const { return(false); }
      virtual void Stop()            {}
      // Audio sample manipulation functions
      virtual void AdjustAmplitude(float fLeftGain, float fRightGain);
      virtual void ConvertMonoToStereo(bool bSilentRightChannel = false);
      virtual void ConvertStereoToMono() {}
      virtual void Convert16BitTo8Bit()  {}
      virtual void SwapLRChannels()      {}
      // I/O functions
      virtual void LoadRAW(const CDataSource &source, ERawFormat eFormat, int32_t n32Frequency);
      virtual void LoadWAV(const CDataSource &source);
      virtual void LoadVSW(const CDataSource &source);
      virtual void SaveVSW(const CDataSink &sink) const;

      // Friend classes
      friend class CDummyAudioEnv;

    protected:
      // Make sure user can't construct this class directly
      inline CDummySoundBuffer() {}
  };

} // namespace vge
#endif
