// ############################################################################################
//  Random.cc
//  Implementation of random number generator classes
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include "../Base/MessageDisplayer.h"
#include "Random.h"


namespace vge {

//
// CRandomCoreUInt32 implementation
//
void CRandomCoreUInt32::Reset(uint32_t seed)
{
  _rnSeed  = seed;
  _srIndex = 0;

  _shiftReg[0] = _rnSeed;
  for(size_t i = 1; i < SR_Size; ++i)
    _shiftReg[i] = SR_MultFac * _shiftReg[i - 1] + 1;
}

void CRandomCoreUInt32::SetState(uint32_t newSeed, uint32_t newSRIndex, const vector<uint32_t> &newShiftReg)
{
  vassert(newShiftReg.size() == SR_Size,
          "The shift-register state array must consist of %d elements", SR_Size);

  _rnSeed   = newSeed;
  _srIndex  = newSRIndex;
  _shiftReg = newShiftReg;
}

void CRandomCoreUInt32::GetState(uint32_t &dstSeed, uint32_t &dstSRIndex, vector<uint32_t> &dstShiftReg)
{
  dstSeed     = _rnSeed;
  dstSRIndex  = _srIndex;
  dstShiftReg = _shiftReg;
}

uint32_t CRandomCoreUInt32::_Generate_()
{
  _srIndex            = (_srIndex + 1) % SR_Size;
  _shiftReg[_srIndex] = _shiftReg[(_srIndex + SR_Size - SR_TapPos - 1) % SR_Size] +
                        _shiftReg[(_srIndex + SR_Size             - 1) % SR_Size];
  return(_shiftReg[_srIndex]);
}

} // namespace vge
