/** \file DialogMacros.h */
// ############################################################################################
//  DialogMacros.h
//  Definition of various macros used by VortexGE dialogs & controls
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _VIDEO_ENVIRONMENT___DIALOG_MACROS_H_
#define _VIDEO_ENVIRONMENT___DIALOG_MACROS_H_


//
// CDialogPage macros
//
//!
//! \ingroup Dialog_Macros
//! \hideinitializer
//! To implement all setters for dialog page's event handlers (keyboard shortcuts
//! and connectable controls) in any class derived from the vge::CDialogPage class.
//!
//! \deprecated This dialog macro is deprecated and will be removed soon (currently it is implemented as dummy so that older code will still compile).
//!
//! \api deprecated
#define CDP_IMPLEMENT_EH(PageName)

//!
//! \ingroup Dialog_Macros
//! \hideinitializer
//! To expand the given class name and control event handler function name
//! to a pointer to member function, usable for classes derived from the
//! vge::CDialogPage class.
//!
//! \api experiment
#define CDP_SLOT(CName, MFName) &CName::MFName

//!
//! \ingroup Dialog_Macros
//! \hideinitializer
//! To delegate a keyboard shortcut's event handler to a control's event handler.
//! This macro can be used to implement a simple keyboard shortcut's event handler
//! which will call the same event handler which also connected to the control via
//! Connect().
//!
//! \api preliminary experiment
#define CDP_KSC_EVENT_DELEGATE(KscEH, CtrlInst, CtrlEH) inline void KscEH() { if(CtrlInst.GetEnable()) CtrlEH(&CtrlInst); }


//
// Dialog macros for string controls
//
//!
//! \internal
//! \ingroup Dialog_Macros
//! \hideinitializer
//! Helper macro to make the _CStringControl member functions inaccessible
//! by users from its derivative classes.
//!
//! <b>Note that this macro will be undefined using \#undef (thus become unusable
//!  by one's applications), unless before including \ref VMain or VMain.h one has
//! already define the macro _KEEP_INTERNAL_MACROS_DEFINED.</b>
//!
//! \api beta
#define _CSTRINGCONTROL_HIDE_MEMBERS()    \
      using _CStringControl::ClearString; \
      using _CStringControl::SetString;   \
      using _CStringControl::GetString;   \
      using _CStringControl::StringLen;


//
// Dialog macros for layout controls
//
//!
//! \ingroup Dialog_Macros
//! \hideinitializer
//! Expand the pointer and block index of the given layout control instance.
//!
//! \api experiment
#define LAYOUT_EXPAND_PI(LCI, LBI) &LCI, LBI
//!
//! \ingroup Dialog_Macros
//! \hideinitializer
//! Expand the pointer, top-left position of the given layout control instance.
//!
//! \api experiment
#define LAYOUT_EXPAND_PXY(LCI, LBI) &LCI, LCI[LBI].x1, LCI[LBI].y1
//!
//! \ingroup Dialog_Macros
//! \hideinitializer
//! Expand the pointer, top-left position and width of the given layout control
//! instance.
//!
//! \api experiment
#define LAYOUT_EXPAND_PXYW(LCI, LBI) &LCI, LCI[LBI].x1, LCI[LBI].y1, LCI[LBI].w
//!
//! \ingroup Dialog_Macros
//! \hideinitializer
//! Expand the pointer, center-left position and width of the given layout control
//! instance.
//!
//! \api experiment
#define LAYOUT_EXPAND_PXYcW(LCI, LBI) &LCI, LCI[LBI].x1, (LCI[LBI].y1 + LCI[LBI].y2) / 2, LCI[LBI].w
//!
//! \ingroup Dialog_Macros
//! \hideinitializer
//! Expand the pointer, top-left position and height of the given layout control
//! instance.
//!
//! \api experiment
#define LAYOUT_EXPAND_PXYH(LCI, LBI) &LCI, LCI[LBI].x1, LCI[LBI].y1, LCI[LBI].h
//!
//! \ingroup Dialog_Macros
//! \hideinitializer
//! Expand the pointer, top-center position and height of the given layout control
//! instance.
//!
//! \api experiment
#define LAYOUT_EXPAND_PXcYH(LCI, LBI) &LCI, (LCI[LBI].x1 + LCI[LBI].x2) / 2, LCI[LBI].y1, LCI[LBI].h
//!
//! \ingroup Dialog_Macros
//! \hideinitializer
//! Expand the pointer, top-left position and size of the given layout control
//! instance.
//!
//! \api experiment
#define LAYOUT_EXPAND_PXYWH(LCI, LBI) &LCI, LCI[LBI].x1, LCI[LBI].y1, LCI[LBI].w, LCI[LBI].h

#endif
