/** \file VX11Color.h */
// ############################################################################################
//  VX11Color.h
//  Prototype & implementation of named color functions
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// NOTE: This file is meant to be included from within the
//       X11 video system implementation


#ifndef _VIDEO_ENVIRONMENT___VID_X11___VX11_COLOR_H_
#define _VIDEO_ENVIRONMENT___VID_X11___VX11_COLOR_H_

// VortexGE header
#include "../Color.h"

namespace vge {

  // Color converters
  //!
  //! Color converters for X11.
  //!
  //! \api pretty stable
  inline uint32_t X11_RGB24(uint8_t un8R, uint8_t un8G, uint8_t un8B)
  { return((uint32_t(un8R) << 16) | (uint32_t(un8G) << 8) | uint32_t(un8B)); }
  //!
  //! Color converters for X11.
  //!
  //! \api pretty stable
  inline uint16_t X11_RGB16(uint8_t un8R, uint8_t un8G, uint8_t un8B, bool bRGB555)
  {
    uint16_t col;
    if(bRGB555)
      col = (uint16_t(un8R & 0xF8) << 7) | (uint16_t(un8G & 0xFC) << 2) | uint16_t(un8B >> 3);
    else
      col = (uint16_t(un8R & 0xF8) << 8) | (uint16_t(un8G & 0xFC) << 3) | uint16_t(un8B >> 3);
    return(col);
  }

} // namespace vge
#endif
