// ############################################################################################
//  mcatedit-MCatEdit-Main.cc
//  Implementation of mcatedit's CMCatEdit and its dialog page classes
//  (common functions)
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// mcatedit's headers
#include "mcatedit-MCatEdit.h"
#include "mcatedit-WIcon.h"

// C++ headers
#include <sstream>


//
// Our dialog page: Directory Selection
//
CPage_SelDir::CPage_SelDir(CPage_Main &mainPage, const string &startDir)
: CDialogPage(mainPage.GetAppInst(), &mainPage.GetAppInst()),
  _app       (mainPage.GetAppInst()),
  _mPage     (mainPage),
  _startDir  (startDir)
{
  _vStrg.Initialize(_iVideoEnv, 1);
}

CPage_SelDir::~CPage_SelDir() throw()
{
}

void CPage_SelDir::SetParam(PFNToInvoke pfnTI, const string &infoText)
{
  _pfnTI = pfnTI;
  lblSDirInfo.SetString(infoText.c_str());
  lblSDirPath.SetString(_listDir().c_str());
}

void CPage_SelDir::InitializeAllControls()
{
  InitControl(pbrSDir, NULL, 0, 0, _iVideoEnv->WindowWidth(), _iVideoEnv->WindowHeight(), _M("Select Directory"));
  InitControl(hlySDir, &pbrSDir, CGLayout::DEFAULT_PAD_SIZE, 0);
      hlySDir.AddBlocksEx(5, 20, 20, 0, 5, 25);
  InitControl(lblSDirInfo, hlySDir, 0, "<description>");
  InitControl(lblSDirPath, hlySDir, 1, "<selected directory>");
  InitControl(lstSDir, hlySDir, 2);
      Connect(lstSDir, CDP_SLOT(CPage_SelDir, _onLstSDir));
  InitControl(vlySDir, hlySDir, 4, 0);
      vlySDir.AddBlocks(2);
  InitControl(btnSDirOK, vlySDir, 0, _M("OK"), false, true);
      Connect(btnSDirOK, CDP_SLOT(CPage_SelDir, _onBtnOK));
  InitControl(btnSDirCancel, vlySDir, 1, _M("Cancel"), true, false);
      Connect(btnSDirCancel, CDP_SLOT(CPage_SelDir, _onBtnCancel));
}

const string CPage_SelDir::_listDir(const char *path)
{
  // For storing the current (last) directory path
  static string curDir = _startDir;

  // Make sure that current directory is still valid
  try {
    VSetCWD(curDir.c_str());
  }
  catch(...) {
    try {
      curDir = VGetCWD();
      VSetCWD(curDir.c_str());
    }
    catch(...) {
      curDir = "/";
      VSetCWD(curDir.c_str());
    }
  }

  // Update the new directory path
  string newDir = path ? (curDir + string(path) + string("/")) : curDir;
  try {
    VSetCWD(newDir.c_str());
    newDir = VGetCWD();
  }
  catch(...) {}
  VSetCWD(curDir.c_str());

  // Update the directory list
  try {
    // List and sort the contents of the new directory
    vector<string> list0, list1, list2;
    VListDirectory(list0, newDir.c_str());
    // Separate and sort the list
    for(size_t i = 0; i < list0.size(); ++i) {
      string &e = list0[i];
      if(e=="." || e=="..")
        continue;
      else if(e[0] == '.')
        list2.push_back(e);
      else
        list1.push_back(e);
    }
    stable_sort(list1.begin(), list1.end());
    stable_sort(list2.begin(), list2.end());
    // Update the current directory path with the new one
    if(newDir == "//") curDir = "/";
    else               curDir = newDir;
    // Clear the old list
    lstSDir.DelAllItems();
    // Put the ".." entry on the list first
    lstSDir.AddItem("..");
    // Walk trough the 1st list and fill in the list
    for(size_t i = 0; i < list1.size(); ++i) {
      if(list1[i] == "INDEX.cat") {
        lstSDir.AddItem(list1[i]);
        continue;
      }
      TFileInfo fi;
      try {
        VGetFileInfo(fi, newDir + list1[i]);
        if(fi.Type.IsDirectory) lstSDir.AddItem(list1[i].c_str());
      }
      catch(...) {}
    }
    // Walk trough the 2nd list and fill in the list
    for(size_t i = 0; i < list2.size(); ++i) {
      TFileInfo fi;
      try {
        VGetFileInfo(fi, newDir + list2[i]);
        if(fi.Type.IsDirectory) lstSDir.AddItem(list2[i].c_str());
      }
      catch(...) {}
    }
  }
  catch(...) {}

  // Return the current directory
  return(curDir);
}




//
// Our dialog page: Locale Management
//
CPage_MLoc::CPage_MLoc(CPage_Main &mainPage)
: CDialogPage(mainPage.GetAppInst(), &mainPage.GetAppInst()),
  _app       (mainPage.GetAppInst()),
  _mPage     (mainPage)
{
  _vStrg.Initialize(_iVideoEnv, 1);
}

CPage_MLoc::~CPage_MLoc() throw()
{
}

void CPage_MLoc::InitializeAllControls()
{
  InitControl(pbrLoc, NULL, 0, 0, _iVideoEnv->WindowWidth(), _iVideoEnv->WindowHeight(), _M("Manage Locales"));
  InitControl(hlyLoc, &pbrLoc);
      hlyLoc.AddBlocksEx(3, 25, 0, 25);
  InitControl(vlyLoc, hlyLoc, 0, 0);
      vlyLoc.AddBlocks(2);
  InitControl(btnLocAdd, vlyLoc, 0, _M("New"));
      Connect(btnLocAdd, CDP_SLOT(CPage_MLoc, _onBtnNew));
  InitControl(btnLocDel, vlyLoc, 1, _M("Delete"));
      Connect(btnLocDel, CDP_SLOT(CPage_MLoc, _onBtnDel));
  InitControl(lstLoc, hlyLoc, 1);
      Connect(lstLoc, CDP_SLOT(CPage_MLoc, _onLstLoc));
  InitControl(btnLocDone, hlyLoc, 2, _M("Done"), true, true);
      Connect(btnLocDone, CDP_SLOT(CPage_MLoc, _onBtnDone));
}

#ifndef __USE_UTF8_DIALOG_STRING__
bool CPage_MLoc::_sltbCallback(ETBEvent eventCode, CVideoEnv &instVideoEnv,
                              CSLTextBox &slTextBox, string &editBuffer,
                              string *pasteBuffer, char &charCode,
                              EKeySymbol keySym)
#else
bool CPage_MLoc::_sltbCallback(ETBEvent eventCode, CVideoEnv &instVideoEnv,
                              CSLTextBox &slTextBox, CUString &editBuffer,
                              CUString *pasteBuffer, CUString::Utf32 &charCode,
                              EKeySymbol keySym)
#endif
{
  UNUSED_PARAMS_5(instVideoEnv, slTextBox, editBuffer, pasteBuffer, keySym);

  if(eventCode == ETBE_Paste) {
    return(false);
  }
  else if(eventCode == ETBE_AddChar) {
    //! todo Add code for validating the locale name
    if(isalnum(charCode)) return(true);
    if(charCode == '_') return(true);
    if(charCode == '.') return(true);
    if(charCode == '-') return(true);
    if(charCode == '@') return(true);
    return(false);
  }
  return(true);
}




//
// Our dialog page: Find in ID or MT
//
CPage_Find::CPage_Find(CPage_Main &mainPage)
: CDialogPage(mainPage.GetAppInst(), &mainPage.GetAppInst()),
  _app       (mainPage.GetAppInst()),
  _mPage     (mainPage)
{
  _vStrg.Initialize(_iVideoEnv, 1);

  RS_TITLE = _M("Result");
}

CPage_Find::~CPage_Find() throw()
{
}

void CPage_Find::SetParam(bool findID)
{
  rdoFindOptID.SetValue(findID);
  rdoFindOptMT.SetValue(!findID);

  SetParam(0, 0, "");
}

void CPage_Find::SetParam(uint32_t cur, uint32_t tot, const string &text)
{
  const bool nf = !tot;

  txtFindRes.SetEnable(!nf);
  btnFindResFirst.SetEnable(!nf);
  btnFindResPrev.SetEnable(!nf);
  btnFindResNext.SetEnable(!nf);
  btnFindResLast.SetEnable(!nf);

  if(nf) {
    string str  = RS_TITLE;
           str += " #0/0";
    lblFindResR.SetString(str.c_str());
    txtFindRes.SetText("");
  }
  else {
    stringstream ss;
    ss << RS_TITLE << " #" << cur + 1 << "/" << tot;
    lblFindResR.SetString(ss.str().c_str());
    txtFindRes.SetText(text.c_str());
  }
}

void CPage_Find::InitializeAllControls()
{
  // Main section
  InitControl(hlyFind, NULL, 0, 0);
      hlyFind.AddBlocksEx(2, 200, 0);
  InitControl(pbrFindOpt, hlyFind, 0, _M("Find Word(s)"));
  InitControl(pnlFindRes, hlyFind, 1);

  // Setting section
  InitControl(hlyFindOpt, &pbrFindOpt, CGLayout::DEFAULT_PAD_SIZE, 0);
      hlyFindOpt.AddBlocksEx(3, 0, 18, 25);
  InitCompound(vhgFindOpt, hlyFindOpt, 0, 0, 0);
      vhgFindOpt.AddCols(2);
      vhgFindOpt[0].AddBlocksEx(5, 18, 18, 18, 18, 0);
      vhgFindOpt[1].AddBlocksEx(7, 18, 18, 18, 18, 5, 18, 0);
  // In vhgFindOpt[0]
  InitControl(lblFindOptFI, vhgFindOpt[0], 0, _M("Find in:"));
  InitControl(rdoFindOptID, vhgFindOpt[0], 1, 15, _M("Original messages (IDs/keys)"));
  InitControl(rdoFindOptMT, vhgFindOpt[0], 2, 15, _M("Translated messages"));
  GroupRadioButtons(&rdoFindOptID, &rdoFindOptMT, NULL);
  // In vhgFindOpt[1]
  InitControl(lblFindOptMF, vhgFindOpt[1], 0, _M("Matching rule:"));
  InitControl(rdoFindOptSingle, vhgFindOpt[1], 1, 15, _M("Contains at least one word"));
  InitControl(rdoFindOptAll, vhgFindOpt[1], 2, 15, _M("Contains all words"));
      rdoFindOptAll.SetValue(true);
  InitControl(rdoFindOptExact, vhgFindOpt[1], 3, 15, _M("Contains exact words sequence"));
  GroupRadioButtons(&rdoFindOptSingle, &rdoFindOptAll, &rdoFindOptExact, NULL);
  InitControl(chkFindOptCS, vhgFindOpt[1], 5, 15, _M("Case sensitive"));
      chkFindOptCS.SetValue(true);
  // In hlyFindOpt
  InitControl(lblFindOptTF, hlyFindOpt, 1, _M("Word(s) to find:"));
  InitControl(vlyFindOpt, hlyFindOpt, 2, 0, 0);
      vlyFindOpt.AddBlocksEx(3, 0, 5, 80);
  InitControl(txtFindOpt, vlyFindOpt, 0);
  InitControl(btnFindOptSearch, vlyFindOpt, 2, _M("Search"), false, true);
      Connect(btnFindOptSearch, CDP_SLOT(CPage_Find, _onBtnSearch));

  // Result section
  InitControl(hlyFindRes, &pnlFindRes);
      hlyFindRes.AddBlocksEx(3, 15, 0, 25);
  InitControl(lblFindResR, hlyFindRes, 0, RS_TITLE.c_str());
  InitControl(txtFindRes, hlyFindRes, 1);
      txtFindRes.SetReadOnly(true);
  InitControl(vlyFindRes, hlyFindRes, 2, 0, 0);
      vlyFindRes.AddBlocks(6);
  InitControl(btnFindResFirst, vlyFindRes, 0, "<<");
      Connect(btnFindResFirst, CDP_SLOT(CPage_Find, _onBtnFirst));
  InitControl(btnFindResPrev, vlyFindRes, 1, "<");
      Connect(btnFindResPrev, CDP_SLOT(CPage_Find, _onBtnPrev));
  InitControl(btnFindResNext, vlyFindRes, 2, ">");
      Connect(btnFindResNext, CDP_SLOT(CPage_Find, _onBtnNext));
  InitControl(btnFindResLast, vlyFindRes, 3, ">>");
      Connect(btnFindResLast, CDP_SLOT(CPage_Find, _onBtnLast));
  InitControl(btnFindResDone, vlyFindRes, 5, _M("Done"), true, false);
      Connect(btnFindResDone, CDP_SLOT(CPage_Find, _onBtnDone));
}

void CPage_Find::_updateSearchDisplay()
{
  TSearchResult sr;
  _app.pageMain._theCatalog.Search_GetResult(sr);

  SetParam(sr.resultCurIndex, sr.resultTotIndex, sr.fullText);

  RedrawAllControls();
  _iVideoEnv->UpdateDisplay();
}




//
// Our dialog page: Main Page
//
CPage_Main::CPage_Main(CMCatEdit &appClass)
: CDialogPage(appClass, &appClass),
  _app       (appClass)
{
  _vStrg.Initialize(_iVideoEnv, 1);

  ID_TITLE = _M("Original Message (ID/Key)");
  MT_TITLE = _M("Translated Message");
}

CPage_Main::~CPage_Main() throw()
{
}

void CPage_Main::UpdateAllControlsEnableStates()
{
  // Menu bar section
  btnMNSave.SetEnable(false);
  btnMNSaveAs.SetEnable(false);
  btnMNIndexInfo.SetEnable(false);
  btnMNLocale.SetEnable(false);
  // Message ID section
  txtID.SetText("");
  txtID.SetEnable(false);
  btnIDAdd.SetEnable(false);
  btnIDDel.SetEnable(false);
  btnIDFirst.SetEnable(false);
  btnIDPrev.SetEnable(false);
  btnIDNext.SetEnable(false);
  btnIDLast.SetEnable(false);
  btnIDFind.SetEnable(false);
  // Message translation section
  txtMT.SetText("");
  txtMT.SetEnable(false);
  btnMTFirst.SetEnable(false);
  btnMTPrev.SetEnable(false);
  btnMTNext.SetEnable(false);
  btnMTLast.SetEnable(false);
  btnMTFind.SetEnable(false);

  // Just return if the catalog is not open
  if(!_theCatalog.IsOpen()) {
    lblID.SetString(ID_TITLE.c_str());
    lblMT.SetString(MT_TITLE.c_str());
    RedrawAllControls();
    return;
  }


  // Menu bar section
  btnMNSave.SetEnable(true);
  btnMNSaveAs.SetEnable(true);
  btnMNIndexInfo.SetEnable(true);
  btnMNLocale.SetEnable(true);

  // Message ID section
  btnIDAdd.SetEnable(true);
  if(_theCatalog.HasNoKey())
    lblID.SetString(ID_TITLE.c_str());
  else {
    stringstream ss;
    ss << ID_TITLE
       << " #" << (_theCatalog.GetCurrentKeyIndex() + 1) << "/" << _theCatalog.GetTotalStoredKeys()
       << " [ni=" << _theCatalog.GetCurrentKeyID() << "]";
    lblID.SetString(ss.str().c_str());

    txtID.SetText(_theCatalog.GetCurrentKeyText().c_str());
    txtID.SetEnable(true);
    btnIDDel.SetEnable(true);
    if(_theCatalog.GetCurrentKeyIndex()) {
      btnIDFirst.SetEnable(true);
      btnIDPrev.SetEnable(true);
    }
    if(_theCatalog.GetCurrentKeyIndex() < _theCatalog.GetLastKeyIndex()) {
      btnIDNext.SetEnable(true);
      btnIDLast.SetEnable(true);
    }
    btnIDFind.SetEnable(true);
  }

  // Message translation section
  if(_theCatalog.HasNoTranslation() || _theCatalog.HasNoKey())
    lblMT.SetString(MT_TITLE.c_str());
  else {
    stringstream ss;
    ss << MT_TITLE
       << " - " << _theCatalog.GetCurrentTranslationTableLocale()
       << " [ni=" << _theCatalog.GetCurrentTranslationTableID() << "]";
    lblMT.SetString(ss.str().c_str());
    txtMT.SetEnable(true);
    txtMT.SetText(_theCatalog.GetCurrentTranslationTableText().c_str());
    btnMTFirst.SetEnable(true);
    btnMTPrev.SetEnable(true);
    btnMTNext.SetEnable(true);
    btnMTLast.SetEnable(true);
    btnMTFind.SetEnable(true);
  }

  RedrawAllControls();
}

void CPage_Main::InitializeAllControls()
{
  // Main section
  InitControl(hlyMain, NULL, 0, 0);
      hlyMain.AddBlocksEx(4, 25, 0, 0, 25);
  InitControl(vlyMenu, hlyMain, 0, 0, 0);
      vlyMenu.AddBlocks(6);
  InitControl(pnlID, hlyMain, 1);
  InitControl(pnlMT, hlyMain, 2);
  InitControl(stbSB, hlyMain, 3, "");

  // Menu bar section
  InitControl(btnMNNew, vlyMenu, 0, _M("New"), false, false);
      Connect(btnMNNew, CDP_SLOT(CPage_Main, _onMnuFileNew));
      btnMNNew.SetTooltip(_M("Create a new message catalog"));
  InitControl(btnMNOpen, vlyMenu, 1, _M("Open"), false, false);
      Connect(btnMNOpen, CDP_SLOT(CPage_Main, _onMnuFileOpen));
      btnMNOpen.SetTooltip(_M("Open an existing message catalog"));
  InitControl(btnMNSave, vlyMenu, 2, _M("Save"), false, false);
      Connect(btnMNSave, CDP_SLOT(CPage_Main, _onMnuFileSave));
      btnMNSave.SetTooltip(_M("Save the current message catalog"));
  InitControl(btnMNSaveAs, vlyMenu, 3, _M("Save As"), false, false);
      Connect(btnMNSaveAs, CDP_SLOT(CPage_Main, _onMnuFileSaveAs));
      btnMNSaveAs.SetTooltip(_M("Save the current message catalog to a different directory"));
  InitControl(btnMNIndexInfo, vlyMenu, 4, _M("Index Info"), false, false);
      Connect(btnMNIndexInfo, CDP_SLOT(CPage_Main, _onMnuFileIndexInfo));
      btnMNIndexInfo.SetTooltip(_M("Edit additional information of INDEX.cat"));
  InitControl(btnMNLocale, vlyMenu, 5, _M("Locales"), false, false);
      Connect(btnMNLocale, CDP_SLOT(CPage_Main, _onMnuFileLocale));
      btnMNLocale.SetTooltip(_M("Manage locales (translation tables)"));

  // Message ID section
  InitControl(hlyID, &pnlID);
      hlyID.AddBlocksEx(3, 15, 0, 25);
  InitControl(lblID, hlyID, 0, ID_TITLE);
  InitControl(txtID, hlyID, 1);
  InitControl(vlyID, hlyID, 2, 0, 0);
      vlyID.AddBlocks(9);
  InitControl(btnIDAdd, vlyID, 0, "+");
      Connect(btnIDAdd, CDP_SLOT(CPage_Main, _onBtnIDAdd));
      btnIDAdd.SetTooltip(_M("Create a new ID/key"));
  InitControl(btnIDDel, vlyID, 1, "-");
      Connect(btnIDDel, CDP_SLOT(CPage_Main, _onBtnIDDel));
      btnIDDel.SetTooltip(_M("Delete current ID/key"));
  InitControl(btnIDFirst, vlyID, 3, "<<");
      Connect(btnIDFirst, CDP_SLOT(CPage_Main, _onBtnIDFirst));
      btnIDFirst.SetTooltip(_M("Go to the first ID/key"));
  InitControl(btnIDPrev, vlyID, 4, "<");
      Connect(btnIDPrev, CDP_SLOT(CPage_Main, _onBtnIDPrev));
      btnIDPrev.SetTooltip(_M("Go to the previous ID/key"));
  InitControl(btnIDNext, vlyID, 5, ">");
      Connect(btnIDNext, CDP_SLOT(CPage_Main, _onBtnIDNext));
      btnIDNext.SetTooltip(_M("Go to the next ID/key"));
  InitControl(btnIDLast, vlyID, 6, ">>");
      Connect(btnIDLast, CDP_SLOT(CPage_Main, _onBtnIDLast));
      btnIDLast.SetTooltip(_M("Go to the last ID/key"));
  InitControl(btnIDFind, vlyID, 8, _M("Find"));
      Connect(btnIDFind, CDP_SLOT(CPage_Main, _onBtnIDFind));
      btnIDFind.SetTooltip(_M("Search in original messages (IDs/keys)"));

  // Message translation section
  InitControl(hlyMT, &pnlMT);
      hlyMT.AddBlocksEx(3, 15, 0, 25);
  InitControl(lblMT, hlyMT, 0, MT_TITLE);
  InitControl(txtMT, hlyMT, 1);
  InitControl(vlyMT, hlyMT, 2, 0, 0);
      vlyMT.AddBlocks(9);
  InitControl(btnMTFirst, vlyMT, 3, "<< ?");
      Connect(btnMTFirst, CDP_SLOT(CPage_Main, _onBtnMTFirst));
      btnMTFirst.SetTooltip(_M("Search and go to the first untranslated message"));
  InitControl(btnMTPrev, vlyMT, 4, "< ?");
      Connect(btnMTPrev, CDP_SLOT(CPage_Main, _onBtnMTPrev));
      btnMTPrev.SetTooltip(_M("Search and go to the previous untranslated message"));
  InitControl(btnMTNext, vlyMT, 5, "> ?");
      Connect(btnMTNext, CDP_SLOT(CPage_Main, _onBtnMTNext));
      btnMTNext.SetTooltip(_M("Search and go to the next untranslated message"));
  InitControl(btnMTLast, vlyMT, 6, ">> ?");
      Connect(btnMTLast, CDP_SLOT(CPage_Main, _onBtnMTLast));
      btnMTLast.SetTooltip(_M("Search and go to the last untranslated message"));
  InitControl(btnMTFind, vlyMT, 8, _M("Find"));
      Connect(btnMTFind, CDP_SLOT(CPage_Main, _onBtnMTFind));
      btnMTFind.SetTooltip(_M("Search in translated messages"));

  // Menu bar section
  InitControl(btnMNAbout, vlyMT, 0, _M("About"), false, false);
      Connect(btnMNAbout, CDP_SLOT(CPage_Main, _onMnuFileAbout));
      btnMNAbout.SetTooltip(_M("About \"VortexGE Message Catalog Editor\""));
}

void CPage_Main::_saveCurValues()
{
  if(!_theCatalog.HasNoKey()) {
    // Check if the text box does really contains non empty text
    const string ttxt  = txtID.GetText();
          bool   empty = true;
    for(size_t i = 0; i < ttxt.length(); ++i) {
      if(isspace(ttxt[i])) continue;
      empty = false;
      break;
    }
    // Put the text
    if(!empty)
      _theCatalog.SetCurrentKeyText(txtID.GetText());
    else
      _theCatalog.SetCurrentKeyText("");
  }

  // Get the translated message
  {
    // Check if the text box does really contains non empty text
    string ttxt  = txtMT.GetText();
    bool   empty = true;
    for(size_t i = 0; i < ttxt.length(); ++i) {
      if(isspace(ttxt[i])) continue;
      empty = false;
      break;
    }
    if(empty) ttxt.clear();
    // Put the text
    if(!_theCatalog.HasNoTranslation()) _theCatalog.SetCurrentTranslationText(ttxt);
  }
}




//
// Our application class
//
const string CMCatEdit::TEXT_VERSION = "v1.0.9";
const string CMCatEdit::TEXT_YEAR    = "2004-2006";

const string CMCatEdit::PAGE_MAIN = "Main";
const string CMCatEdit::PAGE_SDIR = "SDir";
const string CMCatEdit::PAGE_MLOC = "MLoc";
const string CMCatEdit::PAGE_FIND = "Find";

const uint32_t CMCatEdit::APP_INACTIVE_STORE;
const uint32_t CMCatEdit::PRE_DIALOG_STORE;
const uint32_t CMCatEdit::VSTORAGE_CAPACITY;


CMCatEdit::CMCatEdit(uint32_t wWidth, uint32_t wHeight, const string &startDir, Mode mode)
: CMCatEditBase("", NULL, wWidth, wHeight, 0, 0, EAIM_Disabled, E3DIM_Disabled),
  pageMain      (*this),
  pageSDir      (pageMain, startDir),
  pageMLoc      (pageMain),
  pageFind      (pageMain)
{
  // (Re-)initialize the window with our specific settings
  WINDOW_TITLE = _M("VortexGE Message Catalog Editor");
  SetWindowTitle("");
  iVideoEnv().SetWindowIcon(mcatedit_xpm);
  iVideoEnv().MoveCenterWindow(true);
  //iVideoEnv().SetFullscreen(true);

  // Register the pages (remember that the 1st page will become the main page)
  _dialogPageManager.RegisterPage(pageMain, PAGE_MAIN);
  _dialogPageManager.RegisterPage(pageSDir, PAGE_SDIR);
  _dialogPageManager.RegisterPage(pageMLoc, PAGE_MLOC);
  _dialogPageManager.RegisterPage(pageFind, PAGE_FIND);

  // Initialize video storage
  csVStrg.Initialize(&iVideoEnv(), VSTORAGE_CAPACITY);

  // Initialize dialog
  pageSDir.InitializeAllControls();
  pageMLoc.InitializeAllControls();
  pageFind.InitializeAllControls();
  pageMain.InitializeAllControls();
  pageMain.UpdateAllControlsEnableStates();

  // Do we ordered to create a new catalog in the start directory ?
  if(mode == ModeNew) {
    pageMain._onMnuFileNew(startDir);
    pageMain.UpdateAllControlsEnableStates();
  }

  // Do we ordered to create open an existing catalogin the start directory ?
  if(mode == ModeOpen) {
    pageMain._onMnuFileOpen(startDir);
    pageMain.UpdateAllControlsEnableStates();
  }

  // Update the display for the first time
  iVideoEnv().UpdateDisplay();
}

CMCatEdit::~CMCatEdit() throw()
{
}

void CMCatEdit::SetWindowTitle(const string &addText)
{
  if(addText.empty()) {
    iVideoEnv().SetWindowCaption(WINDOW_TITLE.c_str());
    return;
  }

  const string tmp = WINDOW_TITLE + " - " + addText;
  iVideoEnv().SetWindowCaption(tmp.c_str());
}

void CMCatEdit::SelectPage(const string &pageName)
{
  _dialogPageManager.SelectPage(pageName);
  _dialogPageManager.RedrawPage();
  iVideoEnv().UpdateDisplay();
}
