// ############################################################################################
//  roomedit-DlgPages-GSZ.cc
//  Implementation of roomedit's CPage_GSZ class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// roomedit's headers
#include "roomedit-DlgPages-GZS.h"
#include "roomedit-RoomEdit.h"


//
// Our dialog page: grid size selection
//
CPage_GSZ::CPage_GSZ(CPage_Main &mainPage)
: CDialogPage(mainPage.GetAppInst(), &mainPage.GetAppInst()),
  _app       (mainPage.GetAppInst()),
  _mPage     (mainPage)
{
  _vStrg.Initialize(_iVideoEnv, 1);
}

CPage_GSZ::~CPage_GSZ() throw()
{
}

void CPage_GSZ::SetGridSize(const TFloatXYZ &gridSize)
{
  spnSz[0].SetValue(int32_t(gridSize.x));
  spnSz[1].SetValue(int32_t(gridSize.y));
  spnSz[2].SetValue(int32_t(gridSize.z));
}

void CPage_GSZ::_onBtnOK(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  float x, y, z;
  if(chkEqu.GetValue())
    x = y = z = spnSz[0].GetValue();
  else {
    x = spnSz[0].GetValue();
    y = spnSz[1].GetValue();
    z = spnSz[2].GetValue();
  }

  (_mPage.*_pfnTI)(TFloatXYZ(x, y, z));
  _app.SelectPage(CRoomEdit::PAGE_MAIN);
}

void CPage_GSZ::_onBtnCancel(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _app.SelectPage(CRoomEdit::PAGE_MAIN);
}

void CPage_GSZ::_onBtnReset(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  const int32_t dgz[] = { int32_t(CPlane3DBuilder::DEFAULT_GRID_SIZE.x),
                          int32_t(CPlane3DBuilder::DEFAULT_GRID_SIZE.y),
                          int32_t(CPlane3DBuilder::DEFAULT_GRID_SIZE.z) };

  for(size_t i = 0; i < 3; ++i) {
    spnSz[i].SetValueProp(10, 500, 10);
    spnSz[i].SetValue(dgz[i]);
    if(i) spnSz[i].SetEnable(false);
    spnSz[i].DrawControl();
  }
  chkEqu.SetValue(true);
  chkEqu.DrawControl();
}

void CPage_GSZ::_onChkEqu(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  for(size_t i = 1; i < 3; ++i) {
    spnSz[i].SetEnable(!chkEqu.GetValue());
    spnSz[i].DrawControl();
  }
}

void CPage_GSZ::InitializeAllControls()
{
  const uint32_t ww = _iVideoEnv->WindowWidth();
  const uint32_t wh = _iVideoEnv->WindowHeight();
  const uint32_t pw = min<uint32_t>(ww, 300);
  const uint32_t ph = min<uint32_t>(wh, 200);
  const uint32_t px = (ww - pw) / 2;
  const uint32_t py = (wh - ph) / 2;

  // Main section
  InitControl(pbrMain, NULL, px, py, pw, ph, _M("Grid Size"));
  InitCompound(hvgMain, &pbrMain);
      hvgMain.AddRowsEx(5, 15, 0, 25, 1, 25);
      hvgMain[0].AddBlocksEx(3, 0, 0, 0);
      hvgMain[1].AddBlocksEx(3, 0, 0, 0);
      hvgMain[2].AddBlocksEx(3, 0, 5, 60);
      hvgMain[4].AddBlocks(2);
  InitControl(btnOK, hvgMain[4], 0, _M("OK"), false, true);
      Connect(btnOK, CDP_SLOT(CPage_GSZ, _onBtnOK));
  InitControl(btnCancel, hvgMain[4], 1, _M("Cancel"), true);
      Connect(btnCancel, CDP_SLOT(CPage_GSZ, _onBtnCancel));

  // Settings section
  const char *szLabel[] = { "X-Axis", "Y-Axis", "Z-Axis" };
  for(size_t i = 0; i < 3; ++i) {
    InitControl(lblSz[i], hvgMain[0], i, _M(szLabel[i]), EHTA_Center);
    InitControl(spnSz[i], hvgMain[1], i);
        spnSz[i].SetValueProp(10, 500, 10);
        spnSz[i].SetValue(100);
        if(i) spnSz[i].SetEnable(false);
  }
  InitControl(chkEqu, hvgMain[2], 0, _M("Equal values for all axises"));
      Connect(chkEqu, CDP_SLOT(CPage_GSZ, _onChkEqu));
      chkEqu.SetValue(true);
  InitControl(btnReset, hvgMain[2], 2, _M("Reset"));
      Connect(btnReset, CDP_SLOT(CPage_GSZ, _onBtnReset));
}

void CPage_GSZ::HandleEvent_OnKeyPress(EKeySymbol keySymbol, const char *str)
{
  UNUSED_PARAM(str);

  if(keySymbol==VKS_ENTER || keySymbol==VKS_KP_ENTER)
    _onBtnOK(NULL);
  else if(keySymbol == VKS_ESC)
    _onBtnCancel(NULL);
}
