// ############################################################################################
//  roomedit-RoomEdit-StdEH.cc
//  Implementation of roomedit's CRoomEdit class (standard event handlers)
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// roomedit's headers
#include "roomedit-RoomEdit.h"


//
// Standard event handlers
//
void CRoomEdit::EventKeyPress(EKeySymbol eKeySym, const char *str)
{
  if(eKeySym==VKS_ESC && _dialogPageManager.ActivePage()==PAGE_MAIN) {
    EventCloseWindow();
    return;
  }

  CRoomEditBase::EventKeyPress(eKeySym, str);
}

void CRoomEdit::EventUpdateFrame(bool bWindowIsActive)
{
  CRoomEditBase::EventUpdateFrame(bWindowIsActive);

  if(bWindowIsActive) {
    if(_dialogPageManager.ActivePage() == PAGE_MAIN) pageMain.HandleMovements();
    usleep(1000);
  }
  else {
    csVStrg.RestoreCurrentView(APP_INACTIVE_STORE);
    iVideoEnv().UpdateDisplay();
  }
}

void CRoomEdit::EventWindowActivated()
{
  CRoomEditBase::EventWindowActivated();

  iVideoEnv().UpdateDisplay();
}

void CRoomEdit::EventWindowDeactivated()
{
  CRoomEditBase::EventWindowActivated();

  iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
  iVideoEnv().UseDefaultBigFont();
  iVideoEnv().DrawText(0, 150, "Inactive State", ETA_CenterScreen);
  iVideoEnv().UseDefaultSmallFont();
  iVideoEnv().UpdateDisplay();

  csVStrg.StoreCurrentView(APP_INACTIVE_STORE);
}

void CRoomEdit::EventCloseWindow()
{
  csVStrg.StoreCurrentView(PRE_DIALOG_STORE);

  iVideoEnv().ApplyBlackScreen();
  if(iVideoEnv().MessageBox(_M("Exit Editor"),
                            _M("Do you really want to exit ?"),
                            EMBI_Question, 
                            EMBB_YesNo, 
                            EMBS_Small) == EMBRV_Yes)
    CRoomEditBase::EventCloseWindow();

  csVStrg.RestoreCurrentView(PRE_DIALOG_STORE);
}
